from rest_framework.permissions import SAFE_METHODS, BasePermission


class IsReadOnly(BasePermission):
    """
    The request is a read-only request.
    """

    def has_permission(self, request, view):
        return request.method in SAFE_METHODS


class IsSuperuser(BasePermission):
    """
    Allows access only to super users.
    """

    def has_permission(self, request, view):
        return request.user and request.user.is_superuser



import inspect

from rest_framework.permissions import BasePermission


class P(BasePermission):
    OR = 'OR'
    AND = 'AND'
    NOT = 'NOT'
    IDENTITY = 'IDENTITY'

    def _run_operator(self, operator, f1, f2):
        if operator == self.IDENTITY:
            return f1()
        if operator == self.NOT:
            return not f1()
        if operator == self.OR:
            return f1() or f2()
        if operator == self.AND:
            return f1() and f2()

        raise NotImplementedError('Operator `{}` is not implemented.'.format(operator))

    def _build_function(self, name):
        # Encapsulate functions to have lazy evaluation
        def f(*args, **kwargs):
            def f1():
                return getattr(self.c1, name)(*args, **kwargs)

            def f2():
                return getattr(self.c2, name)(*args, **kwargs)
            return self._run_operator(self.operator, f1, f2)
        return f

    def __init__(self, c1, c2=None, operator=IDENTITY):
        self.c1 = c1() if inspect.isclass(c1) else c1
        self.c2 = c2() if inspect.isclass(c2) else c2
        self.operator = operator

        self.has_permission = self._build_function('has_permission')
        self.has_object_permission = self._build_function('has_object_permission')

    def __or__(self, other):
        return P(self, other, operator=self.OR)

    def __and__(self, other):
        return P(self, other, operator=self.AND)

    def __invert__(self):
        return P(self, operator=self.NOT)

    def __call__(self):
        return self



__title__ = 'DRF Composable Permissions'
__version__ = '0.1.1'
__author__ = 'Jeffrey de Lange'
__license__ = 'BSD 2-Clause'
__copyright__ = 'Copyright 2016 Jeffrey de Lange'

# Version synonym
VERSION = __version__



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import os
import sys
from setuptools import setup


name = 'drf-composable-permissions'
package = 'drf_composable_permissions'
description = 'Compose your permission classes for django-rest-framework.'
url = 'https://github.com/jgadelange/drf-composable-permissions'
author = 'Jeffrey de Lange'
author_email = 'jeffrey@jeffreydelange.nl'
license = 'BSD'


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]",
                     init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = get_version(package)


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    os.system("python setup.py bdist_wheel upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a {0} -m 'version {0}'".format(version))
    print("  git push --tags")
    sys.exit()


setup(
    name=name,
    version=version,
    url=url,
    license=license,
    description=description,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=[],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
